package com.mysql.jdbc;

import com.mysql.jdbc.StatementImpl;
import com.mysql.jdbc.exceptions.MySQLTimeoutException;
import com.mysql.jdbc.profiler.ProfileEventSink;
import com.mysql.jdbc.profiler.ProfilerEvent;
import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.BatchUpdateException;
import java.sql.ParameterMetaData;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

/* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/ServerPreparedStatement.class */
public class ServerPreparedStatement extends PreparedStatement {
    private static final Constructor JDBC_4_SPS_CTOR;
    protected static final int BLOB_STREAM_READ_BUF_SIZE = 8192;
    private static final byte MAX_DATE_REP_LENGTH = 5;
    private static final byte MAX_DATETIME_REP_LENGTH = 12;
    private static final byte MAX_TIME_REP_LENGTH = 13;
    private boolean detectedLongParameterSwitch;
    private int fieldCount;
    private boolean invalid;
    private SQLException invalidationException;
    private boolean isSelectQuery;
    private Buffer outByteBuffer;
    private BindValue[] parameterBindings;
    private Field[] parameterFields;
    private Field[] resultFields;
    private boolean sendTypesToServer;
    private long serverStatementId;
    private int stringTypeCode;
    private boolean serverNeedsResetBeforeEachExecution;
    protected boolean isCached;
    static Class class$com$mysql$jdbc$ConnectionImpl;
    static Class class$java$lang$String;

    static {
        Class<?> cls;
        Class<?> cls2;
        Class<?> cls3;
        if (Util.isJdbc4()) {
            try {
                Class<?> cls4 = Class.forName("com.mysql.jdbc.JDBC4ServerPreparedStatement");
                Class<?>[] clsArr = new Class[MAX_DATE_REP_LENGTH];
                if (class$com$mysql$jdbc$ConnectionImpl == null) {
                    cls = class$("com.mysql.jdbc.ConnectionImpl");
                    class$com$mysql$jdbc$ConnectionImpl = cls;
                } else {
                    cls = class$com$mysql$jdbc$ConnectionImpl;
                }
                clsArr[0] = cls;
                if (class$java$lang$String == null) {
                    cls2 = class$("java.lang.String");
                    class$java$lang$String = cls2;
                } else {
                    cls2 = class$java$lang$String;
                }
                clsArr[1] = cls2;
                if (class$java$lang$String == null) {
                    cls3 = class$("java.lang.String");
                    class$java$lang$String = cls3;
                } else {
                    cls3 = class$java$lang$String;
                }
                clsArr[2] = cls3;
                clsArr[3] = Integer.TYPE;
                clsArr[4] = Integer.TYPE;
                JDBC_4_SPS_CTOR = cls4.getConstructor(clsArr);
                return;
            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            } catch (NoSuchMethodException e2) {
                throw new RuntimeException(e2);
            } catch (SecurityException e3) {
                throw new RuntimeException(e3);
            }
        }
        JDBC_4_SPS_CTOR = null;
    }

    static Class class$(String x0) {
        try {
            return Class.forName(x0);
        } catch (ClassNotFoundException x1) {
            throw new NoClassDefFoundError(x1.getMessage());
        }
    }

    /* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/ServerPreparedStatement$BatchedBindValues.class */
    static class BatchedBindValues {
        BindValue[] batchedParameterValues;

        BatchedBindValues(BindValue[] paramVals) {
            int numParams = paramVals.length;
            this.batchedParameterValues = new BindValue[numParams];
            for (int i = 0; i < numParams; i++) {
                this.batchedParameterValues[i] = new BindValue(paramVals[i]);
            }
        }
    }

    /* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/ServerPreparedStatement$BindValue.class */
    public static class BindValue {
        long boundBeforeExecutionNum;
        public long bindLength;
        int bufferType;
        byte byteBinding;
        double doubleBinding;
        float floatBinding;
        int intBinding;
        public boolean isLongData;
        public boolean isNull;
        boolean isSet;
        long longBinding;
        short shortBinding;
        public Object value;

        BindValue() {
            this.boundBeforeExecutionNum = 0L;
            this.isSet = false;
        }

        BindValue(BindValue copyMe) {
            this.boundBeforeExecutionNum = 0L;
            this.isSet = false;
            this.value = copyMe.value;
            this.isSet = copyMe.isSet;
            this.isLongData = copyMe.isLongData;
            this.isNull = copyMe.isNull;
            this.bufferType = copyMe.bufferType;
            this.bindLength = copyMe.bindLength;
            this.byteBinding = copyMe.byteBinding;
            this.shortBinding = copyMe.shortBinding;
            this.intBinding = copyMe.intBinding;
            this.longBinding = copyMe.longBinding;
            this.floatBinding = copyMe.floatBinding;
            this.doubleBinding = copyMe.doubleBinding;
        }

        void reset() {
            this.isSet = false;
            this.value = null;
            this.isLongData = false;
            this.byteBinding = (byte) 0;
            this.shortBinding = (short) 0;
            this.intBinding = 0;
            this.longBinding = 0L;
            this.floatBinding = 0.0f;
            this.doubleBinding = 0.0d;
        }

        public String toString() {
            return toString(false);
        }

        public String toString(boolean quoteIfNeeded) {
            if (this.isLongData) {
                return "' STREAM DATA '";
            }
            switch (this.bufferType) {
                case 1:
                    return String.valueOf((int) this.byteBinding);
                case 2:
                    return String.valueOf((int) this.shortBinding);
                case 3:
                    return String.valueOf(this.intBinding);
                case 4:
                    return String.valueOf(this.floatBinding);
                case ServerPreparedStatement.MAX_DATE_REP_LENGTH /* 5 */:
                    return String.valueOf(this.doubleBinding);
                case 7:
                case 10:
                case 11:
                case ServerPreparedStatement.MAX_DATETIME_REP_LENGTH /* 12 */:
                case 15:
                case 253:
                case 254:
                    if (quoteIfNeeded) {
                        return new StringBuffer().append("'").append(String.valueOf(this.value)).append("'").toString();
                    }
                    return String.valueOf(this.value);
                case 8:
                    return String.valueOf(this.longBinding);
                default:
                    if (this.value instanceof byte[]) {
                        return "byte data";
                    }
                    if (quoteIfNeeded) {
                        return new StringBuffer().append("'").append(String.valueOf(this.value)).append("'").toString();
                    }
                    return String.valueOf(this.value);
            }
        }
    }

    private void storeTime(Buffer intoBuf, Time tm) throws SQLException {
        intoBuf.ensureCapacity(9);
        intoBuf.writeByte((byte) 8);
        intoBuf.writeByte((byte) 0);
        intoBuf.writeLong(0L);
        Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
        synchronized (sessionCalendar) {
            Date oldTime = sessionCalendar.getTime();
            try {
                sessionCalendar.setTime(tm);
                intoBuf.writeByte((byte) sessionCalendar.get(11));
                intoBuf.writeByte((byte) sessionCalendar.get(MAX_DATETIME_REP_LENGTH));
                intoBuf.writeByte((byte) sessionCalendar.get(MAX_TIME_REP_LENGTH));
                sessionCalendar.setTime(oldTime);
            } catch (Throwable th) {
                sessionCalendar.setTime(oldTime);
                throw th;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public static ServerPreparedStatement getInstance(ConnectionImpl conn, String sql, String catalog, int resultSetType, int resultSetConcurrency) throws SQLException {
        if (!Util.isJdbc4()) {
            return new ServerPreparedStatement(conn, sql, catalog, resultSetType, resultSetConcurrency);
        }
        try {
            return (ServerPreparedStatement) JDBC_4_SPS_CTOR.newInstance(conn, sql, catalog, Constants.integerValueOf(resultSetType), Constants.integerValueOf(resultSetConcurrency));
        } catch (IllegalAccessException e) {
            throw new SQLException(e.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        } catch (IllegalArgumentException e2) {
            throw new SQLException(e2.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        } catch (InstantiationException e3) {
            throw new SQLException(e3.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        } catch (InvocationTargetException e4) {
            Throwable target = e4.getTargetException();
            if (target instanceof SQLException) {
                throw ((SQLException) target);
            }
            throw new SQLException(target.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public ServerPreparedStatement(ConnectionImpl conn, String sql, String catalog, int resultSetType, int resultSetConcurrency) throws SQLException {
        super(conn, catalog);
        this.detectedLongParameterSwitch = false;
        this.invalid = false;
        this.sendTypesToServer = false;
        this.stringTypeCode = 254;
        this.isCached = false;
        checkNullOrEmptyQuery(sql);
        this.isSelectQuery = StringUtils.startsWithIgnoreCaseAndWs(sql, "SELECT");
        if (this.connection.versionMeetsMinimum(MAX_DATE_REP_LENGTH, 0, 0)) {
            this.serverNeedsResetBeforeEachExecution = !this.connection.versionMeetsMinimum(MAX_DATE_REP_LENGTH, 0, 3);
        } else {
            this.serverNeedsResetBeforeEachExecution = !this.connection.versionMeetsMinimum(4, 1, 10);
        }
        this.useTrueBoolean = this.connection.versionMeetsMinimum(3, 21, 23);
        this.hasLimitClause = StringUtils.indexOfIgnoreCase(sql, "LIMIT") != -1;
        this.firstCharOfStmt = StringUtils.firstNonWsCharUc(sql);
        String statementComment = this.connection.getStatementComment();
        this.originalSql = statementComment == null ? sql : new StringBuffer().append("/* ").append(statementComment).append(" */ ").append(sql).toString();
        if (this.connection.versionMeetsMinimum(4, 1, 2)) {
            this.stringTypeCode = 253;
        } else {
            this.stringTypeCode = 254;
        }
        try {
            serverPrepare(sql);
            setResultSetType(resultSetType);
            setResultSetConcurrency(resultSetConcurrency);
            this.parameterTypes = new int[this.parameterCount];
        } catch (SQLException sqlEx) {
            realClose(false, true);
            throw sqlEx;
        } catch (Exception ex) {
            realClose(false, true);
            throw SQLError.createSQLException(ex.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public synchronized void addBatch() throws SQLException {
        checkClosed();
        if (this.batchedArgs == null) {
            this.batchedArgs = new ArrayList();
        }
        this.batchedArgs.add(new BatchedBindValues(this.parameterBindings));
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:31:0x0132
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    @Override // com.mysql.jdbc.PreparedStatement
    protected java.lang.String asSql(boolean r6) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 310
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ServerPreparedStatement.asSql(boolean):java.lang.String");
    }

    /* JADX INFO: Access modifiers changed from: protected */
    @Override // com.mysql.jdbc.StatementImpl
    public void checkClosed() throws SQLException {
        if (this.invalid) {
            throw this.invalidationException;
        }
        super.checkClosed();
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void clearParameters() throws SQLException {
        checkClosed();
        clearParametersInternal(true);
    }

    private void clearParametersInternal(boolean clearServerParameters) throws SQLException {
        boolean hadLongData = false;
        if (this.parameterBindings != null) {
            for (int i = 0; i < this.parameterCount; i++) {
                if (this.parameterBindings[i] != null && this.parameterBindings[i].isLongData) {
                    hadLongData = true;
                }
                this.parameterBindings[i].reset();
            }
        }
        if (clearServerParameters && hadLongData) {
            serverResetStatement();
            this.detectedLongParameterSwitch = false;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void setClosed(boolean flag) {
        this.isClosed = flag;
    }

    @Override // com.mysql.jdbc.PreparedStatement, com.mysql.jdbc.StatementImpl, java.sql.Statement, java.lang.AutoCloseable
    public void close() throws SQLException {
        if (this.isCached) {
            this.isClosed = true;
            this.connection.recachePreparedStatement(this);
        } else {
            realClose(true, true);
        }
    }

    private void dumpCloseForTestcase() {
        StringBuffer buf = new StringBuffer();
        this.connection.generateConnectionCommentBlock(buf);
        buf.append("DEALLOCATE PREPARE debug_stmt_");
        buf.append(this.statementId);
        buf.append(";\n");
        this.connection.dumpTestcaseQuery(buf.toString());
    }

    private void dumpExecuteForTestcase() throws SQLException {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < this.parameterCount; i++) {
            this.connection.generateConnectionCommentBlock(buf);
            buf.append("SET @debug_stmt_param");
            buf.append(this.statementId);
            buf.append("_");
            buf.append(i);
            buf.append("=");
            if (this.parameterBindings[i].isNull) {
                buf.append("NULL");
            } else {
                buf.append(this.parameterBindings[i].toString(true));
            }
            buf.append(";\n");
        }
        this.connection.generateConnectionCommentBlock(buf);
        buf.append("EXECUTE debug_stmt_");
        buf.append(this.statementId);
        if (this.parameterCount > 0) {
            buf.append(" USING ");
            for (int i2 = 0; i2 < this.parameterCount; i2++) {
                if (i2 > 0) {
                    buf.append(", ");
                }
                buf.append("@debug_stmt_param");
                buf.append(this.statementId);
                buf.append("_");
                buf.append(i2);
            }
        }
        buf.append(";\n");
        this.connection.dumpTestcaseQuery(buf.toString());
    }

    private void dumpPrepareForTestcase() throws SQLException {
        StringBuffer buf = new StringBuffer(this.originalSql.length() + 64);
        this.connection.generateConnectionCommentBlock(buf);
        buf.append("PREPARE debug_stmt_");
        buf.append(this.statementId);
        buf.append(" FROM \"");
        buf.append(this.originalSql);
        buf.append("\";\n");
        this.connection.dumpTestcaseQuery(buf.toString());
    }

    /* JADX WARN: Finally extract failed */
    /* JADX WARN: Type inference failed for: r3v4, types: [byte[], byte[][]] */
    @Override // com.mysql.jdbc.PreparedStatement, com.mysql.jdbc.StatementImpl, java.sql.Statement
    public synchronized int[] executeBatch() throws SQLException {
        int[] iArr;
        if (this.connection.isReadOnly()) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.2")).append(Messages.getString("ServerPreparedStatement.3")).toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        checkClosed();
        synchronized (this.connection.getMutex()) {
            clearWarnings();
            BindValue[] oldBindValues = this.parameterBindings;
            try {
                int[] updateCounts = null;
                if (this.batchedArgs != null) {
                    int nbrCommands = this.batchedArgs.size();
                    updateCounts = new int[nbrCommands];
                    if (this.retrieveGeneratedKeys) {
                        this.batchedGeneratedKeys = new ArrayList(nbrCommands);
                    }
                    for (int i = 0; i < nbrCommands; i++) {
                        updateCounts[i] = -3;
                    }
                    SQLException sqlEx = null;
                    BindValue[] previousBindValuesForBatch = null;
                    for (int commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
                        Object arg = this.batchedArgs.get(commandIndex);
                        if (arg instanceof String) {
                            updateCounts[commandIndex] = executeUpdate((String) arg);
                        } else {
                            this.parameterBindings = ((BatchedBindValues) arg).batchedParameterValues;
                            if (previousBindValuesForBatch != null) {
                                for (int j = 0; j < this.parameterBindings.length; j++) {
                                    try {
                                        if (this.parameterBindings[j].bufferType != previousBindValuesForBatch[j].bufferType) {
                                            this.sendTypesToServer = true;
                                            break;
                                        }
                                    } catch (SQLException ex) {
                                        updateCounts[commandIndex] = -3;
                                        if (this.continueBatchOnError) {
                                            sqlEx = ex;
                                        } else {
                                            int[] newUpdateCounts = new int[commandIndex];
                                            System.arraycopy(updateCounts, 0, newUpdateCounts, 0, commandIndex);
                                            throw new BatchUpdateException(ex.getMessage(), ex.getSQLState(), ex.getErrorCode(), newUpdateCounts);
                                        }
                                    }
                                }
                            }
                            try {
                                updateCounts[commandIndex] = executeUpdate(false, true);
                                previousBindValuesForBatch = this.parameterBindings;
                                if (this.retrieveGeneratedKeys) {
                                    ResultSet rs = null;
                                    try {
                                        rs = getGeneratedKeysInternal();
                                        while (rs.next()) {
                                            this.batchedGeneratedKeys.add(new ByteArrayRow(new byte[]{rs.getBytes(1)}));
                                        }
                                        if (rs != null) {
                                            rs.close();
                                        }
                                    } catch (Throwable th) {
                                        if (rs != null) {
                                            rs.close();
                                        }
                                        throw th;
                                    }
                                }
                            } catch (Throwable th2) {
                                BindValue[] bindValueArr = this.parameterBindings;
                                throw th2;
                            }
                        }
                    }
                    if (sqlEx != null) {
                        throw new BatchUpdateException(sqlEx.getMessage(), sqlEx.getSQLState(), sqlEx.getErrorCode(), updateCounts);
                    }
                }
                iArr = updateCounts != null ? updateCounts : new int[0];
            } finally {
                this.parameterBindings = oldBindValues;
                this.sendTypesToServer = true;
                clearBatch();
            }
        }
        return iArr;
    }

    @Override // com.mysql.jdbc.PreparedStatement
    protected ResultSetInternalMethods executeInternal(int maxRowsToRetrieve, Buffer sendPacket, boolean createStreamingResultSet, boolean queryIsSelectOnly, Field[] metadataFromCache, boolean isBatch) throws SQLException {
        this.numberOfExecutions++;
        try {
            return serverExecute(maxRowsToRetrieve, createStreamingResultSet, metadataFromCache);
        } catch (SQLException e) {
            sqlEx = e;
            if (this.connection.getEnablePacketDebug()) {
                this.connection.getIO().dumpPacketRingBuffer();
            }
            if (this.connection.getDumpQueriesOnException()) {
                String extractedSql = toString();
                StringBuffer messageBuf = new StringBuffer(extractedSql.length() + 32);
                messageBuf.append("\n\nQuery being executed when exception was thrown:\n\n");
                messageBuf.append(extractedSql);
                sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf.toString());
            }
            throw sqlEx;
        } catch (Exception ex) {
            if (this.connection.getEnablePacketDebug()) {
                this.connection.getIO().dumpPacketRingBuffer();
            }
            SQLException sqlEx = SQLError.createSQLException(ex.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
            if (this.connection.getDumpQueriesOnException()) {
                String extractedSql2 = toString();
                StringBuffer messageBuf2 = new StringBuffer(extractedSql2.length() + 32);
                messageBuf2.append("\n\nQuery being executed when exception was thrown:\n\n");
                messageBuf2.append(extractedSql2);
                sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf2.toString());
            }
            throw sqlEx;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement
    protected Buffer fillSendPacket() throws SQLException {
        return null;
    }

    @Override // com.mysql.jdbc.PreparedStatement
    protected Buffer fillSendPacket(byte[][] batchedParameterStrings, InputStream[] batchedParameterStreams, boolean[] batchedIsStream, int[] batchedStreamLengths) throws SQLException {
        return null;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public BindValue getBinding(int parameterIndex, boolean forLongData) throws SQLException {
        checkClosed();
        if (this.parameterBindings.length == 0) {
            throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.8"), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        int parameterIndex2 = parameterIndex - 1;
        if (parameterIndex2 < 0 || parameterIndex2 >= this.parameterBindings.length) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.9")).append(parameterIndex2 + 1).append(Messages.getString("ServerPreparedStatement.10")).append(this.parameterBindings.length).toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        if (this.parameterBindings[parameterIndex2] == null) {
            this.parameterBindings[parameterIndex2] = new BindValue();
        } else if (this.parameterBindings[parameterIndex2].isLongData && !forLongData) {
            this.detectedLongParameterSwitch = true;
        }
        this.parameterBindings[parameterIndex2].isSet = true;
        this.parameterBindings[parameterIndex2].boundBeforeExecutionNum = this.numberOfExecutions;
        return this.parameterBindings[parameterIndex2];
    }

    byte[] getBytes(int parameterIndex) throws SQLException {
        BindValue bindValue = getBinding(parameterIndex, false);
        if (bindValue.isNull) {
            return null;
        }
        if (bindValue.isLongData) {
            throw new NotImplemented();
        }
        if (this.outByteBuffer == null) {
            this.outByteBuffer = new Buffer(this.connection.getNetBufferLength());
        }
        this.outByteBuffer.clear();
        int originalPosition = this.outByteBuffer.getPosition();
        storeBinding(this.outByteBuffer, bindValue, this.connection.getIO());
        int newPosition = this.outByteBuffer.getPosition();
        int length = newPosition - originalPosition;
        byte[] valueAsBytes = new byte[length];
        System.arraycopy(this.outByteBuffer.getByteBuffer(), originalPosition, valueAsBytes, 0, length);
        return valueAsBytes;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public java.sql.ResultSetMetaData getMetaData() throws SQLException {
        checkClosed();
        if (this.resultFields == null) {
            return null;
        }
        return new ResultSetMetaData(this.resultFields, this.connection.getUseOldAliasMetadataBehavior());
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public ParameterMetaData getParameterMetaData() throws SQLException {
        checkClosed();
        if (this.parameterMetaData == null) {
            this.parameterMetaData = new MysqlParameterMetadata(this.parameterFields, this.parameterCount);
        }
        return this.parameterMetaData;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    @Override // com.mysql.jdbc.PreparedStatement
    public boolean isNull(int paramIndex) {
        throw new IllegalArgumentException(Messages.getString("ServerPreparedStatement.7"));
    }

    /* JADX INFO: Access modifiers changed from: protected */
    @Override // com.mysql.jdbc.PreparedStatement, com.mysql.jdbc.StatementImpl
    public void realClose(boolean calledExplicitly, boolean closeOpenResults) throws SQLException {
        if (!this.isClosed && this.connection != null) {
            if (this.connection.getAutoGenerateTestcaseScript()) {
                dumpCloseForTestcase();
            }
            SQLException exceptionDuringClose = null;
            if (calledExplicitly && !this.connection.isClosed()) {
                synchronized (this.connection.getMutex()) {
                    try {
                        MysqlIO mysql = this.connection.getIO();
                        Buffer packet = mysql.getSharedSendPacket();
                        packet.writeByte((byte) 25);
                        packet.writeLong(this.serverStatementId);
                        mysql.sendCommand(25, null, packet, true, null);
                    } catch (SQLException sqlEx) {
                        exceptionDuringClose = sqlEx;
                    }
                }
            }
            super.realClose(calledExplicitly, closeOpenResults);
            clearParametersInternal(false);
            this.parameterBindings = null;
            this.parameterFields = null;
            this.resultFields = null;
            if (exceptionDuringClose != null) {
                throw exceptionDuringClose;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void rePrepare() throws SQLException {
        this.invalidationException = null;
        try {
            serverPrepare(this.originalSql);
        } catch (SQLException sqlEx) {
            this.invalidationException = sqlEx;
        } catch (Exception ex) {
            this.invalidationException = SQLError.createSQLException(ex.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
        if (this.invalidationException != null) {
            this.invalid = true;
            this.parameterBindings = null;
            this.parameterFields = null;
            this.resultFields = null;
            if (this.results != null) {
                try {
                    this.results.close();
                } catch (Exception e) {
                }
            }
            if (this.connection != null) {
                if (this.maxRowsChanged) {
                    this.connection.unsetMaxRows(this);
                }
                if (!this.connection.getDontTrackOpenResources()) {
                    this.connection.unregisterStatement(this);
                }
            }
        }
    }

    private ResultSetInternalMethods serverExecute(int maxRowsToRetrieve, boolean createStreamingResultSet, Field[] metadataFromCache) throws SQLException {
        ResultSetInternalMethods rs;
        synchronized (this.connection.getMutex()) {
            if (this.detectedLongParameterSwitch) {
                boolean firstFound = false;
                long boundTimeToCheck = 0;
                for (int i = 0; i < this.parameterCount - 1; i++) {
                    if (this.parameterBindings[i].isLongData) {
                        if (firstFound && boundTimeToCheck != this.parameterBindings[i].boundBeforeExecutionNum) {
                            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.11")).append(Messages.getString("ServerPreparedStatement.12")).toString(), SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
                        }
                        firstFound = true;
                        boundTimeToCheck = this.parameterBindings[i].boundBeforeExecutionNum;
                    }
                }
                serverResetStatement();
            }
            for (int i2 = 0; i2 < this.parameterCount; i2++) {
                if (!this.parameterBindings[i2].isSet) {
                    throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.13")).append(i2 + 1).append(Messages.getString("ServerPreparedStatement.14")).toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                }
            }
            for (int i3 = 0; i3 < this.parameterCount; i3++) {
                if (this.parameterBindings[i3].isLongData) {
                    serverLongData(i3, this.parameterBindings[i3]);
                }
            }
            if (this.connection.getAutoGenerateTestcaseScript()) {
                dumpExecuteForTestcase();
            }
            MysqlIO mysql = this.connection.getIO();
            Buffer packet = mysql.getSharedSendPacket();
            packet.clear();
            packet.writeByte((byte) 23);
            packet.writeLong(this.serverStatementId);
            if (this.connection.versionMeetsMinimum(4, 1, 2)) {
                if (this.resultFields != null && this.connection.isCursorFetchEnabled() && getResultSetType() == 1003 && getResultSetConcurrency() == 1007 && getFetchSize() > 0) {
                    packet.writeByte((byte) 1);
                } else {
                    packet.writeByte((byte) 0);
                }
                packet.writeLong(1L);
            }
            int nullCount = (this.parameterCount + 7) / 8;
            int nullBitsPosition = packet.getPosition();
            for (int i4 = 0; i4 < nullCount; i4++) {
                packet.writeByte((byte) 0);
            }
            byte[] nullBitsBuffer = new byte[nullCount];
            packet.writeByte(this.sendTypesToServer ? (byte) 1 : (byte) 0);
            if (this.sendTypesToServer) {
                for (int i5 = 0; i5 < this.parameterCount; i5++) {
                    packet.writeInt(this.parameterBindings[i5].bufferType);
                }
            }
            for (int i6 = 0; i6 < this.parameterCount; i6++) {
                if (!this.parameterBindings[i6].isLongData) {
                    if (!this.parameterBindings[i6].isNull) {
                        storeBinding(packet, this.parameterBindings[i6], mysql);
                    } else {
                        int i7 = i6 / 8;
                        nullBitsBuffer[i7] = (byte) (nullBitsBuffer[i7] | (1 << (i6 & 7)));
                    }
                }
            }
            int endPosition = packet.getPosition();
            packet.setPosition(nullBitsPosition);
            packet.writeBytesNoNull(nullBitsBuffer);
            packet.setPosition(endPosition);
            long begin = 0;
            boolean logSlowQueries = this.connection.getLogSlowQueries();
            boolean gatherPerformanceMetrics = this.connection.getGatherPerformanceMetrics();
            if (this.profileSQL || logSlowQueries || gatherPerformanceMetrics) {
                begin = mysql.getCurrentTimeNanosOrMillis();
            }
            synchronized (this.cancelTimeoutMutex) {
                this.wasCancelled = false;
            }
            StatementImpl.CancelTask timeoutTask = null;
            try {
                if (this.connection.getEnableQueryTimeouts() && this.timeoutInMillis != 0 && this.connection.versionMeetsMinimum(MAX_DATE_REP_LENGTH, 0, 0)) {
                    timeoutTask = new StatementImpl.CancelTask(this);
                    ConnectionImpl connectionImpl = this.connection;
                    ConnectionImpl.getCancelTimer().schedule(timeoutTask, this.timeoutInMillis);
                }
                Buffer resultPacket = mysql.sendCommand(23, null, packet, false, null);
                long queryEndTime = 0;
                if (logSlowQueries || gatherPerformanceMetrics || this.profileSQL) {
                    queryEndTime = mysql.getCurrentTimeNanosOrMillis();
                }
                if (timeoutTask != null) {
                    timeoutTask.cancel();
                    if (timeoutTask.caughtWhileCancelling != null) {
                        throw timeoutTask.caughtWhileCancelling;
                    }
                    timeoutTask = null;
                }
                synchronized (this.cancelTimeoutMutex) {
                    if (this.wasCancelled) {
                        this.wasCancelled = false;
                        throw new MySQLTimeoutException();
                    }
                }
                boolean queryWasSlow = false;
                if (logSlowQueries || gatherPerformanceMetrics) {
                    long elapsedTime = queryEndTime - begin;
                    if (logSlowQueries && elapsedTime >= mysql.getSlowQueryThreshold()) {
                        queryWasSlow = true;
                        StringBuffer mesgBuf = new StringBuffer(48 + this.originalSql.length());
                        mesgBuf.append(Messages.getString("ServerPreparedStatement.15"));
                        mesgBuf.append(mysql.getSlowQueryThreshold());
                        mesgBuf.append(Messages.getString("ServerPreparedStatement.15a"));
                        mesgBuf.append(elapsedTime);
                        mesgBuf.append(Messages.getString("ServerPreparedStatement.16"));
                        mesgBuf.append("as prepared: ");
                        mesgBuf.append(this.originalSql);
                        mesgBuf.append("\n\n with parameters bound:\n\n");
                        mesgBuf.append(asSql(true));
                        this.eventSink.consumeEvent(new ProfilerEvent((byte) 6, "", this.currentCatalog, this.connection.getId(), getId(), 0, System.currentTimeMillis(), elapsedTime, mysql.getQueryTimingUnits(), (String) null, new Throwable(), mesgBuf.toString()));
                    }
                    if (gatherPerformanceMetrics) {
                        this.connection.registerQueryExecutionTime(elapsedTime);
                    }
                }
                this.connection.incrementNumberOfPreparedExecutes();
                if (this.profileSQL) {
                    this.eventSink = ProfileEventSink.getInstance(this.connection);
                    this.eventSink.consumeEvent(new ProfilerEvent((byte) 4, "", this.currentCatalog, this.connectionId, this.statementId, -1, System.currentTimeMillis(), (int) (mysql.getCurrentTimeNanosOrMillis() - begin), mysql.getQueryTimingUnits(), (String) null, new Throwable(), truncateQueryToLog(asSql(true))));
                }
                rs = mysql.readAllResults(this, maxRowsToRetrieve, this.resultSetType, this.resultSetConcurrency, createStreamingResultSet, this.currentCatalog, resultPacket, true, this.fieldCount, metadataFromCache);
                if (this.profileSQL) {
                    long fetchEndTime = mysql.getCurrentTimeNanosOrMillis();
                    this.eventSink.consumeEvent(new ProfilerEvent((byte) 5, "", this.currentCatalog, this.connection.getId(), getId(), 0, System.currentTimeMillis(), fetchEndTime - queryEndTime, mysql.getQueryTimingUnits(), (String) null, new Throwable(), (String) null));
                }
                if (queryWasSlow && this.connection.getExplainSlowQueries()) {
                    String queryAsString = asSql(true);
                    mysql.explainSlowQuery(queryAsString.getBytes(), queryAsString);
                }
                if (!createStreamingResultSet && this.serverNeedsResetBeforeEachExecution) {
                    serverResetStatement();
                }
                this.sendTypesToServer = false;
                this.results = rs;
                if (mysql.hadWarnings()) {
                    mysql.scanForAndThrowDataTruncation();
                }
                if (timeoutTask != null) {
                    timeoutTask.cancel();
                }
            } catch (Throwable th) {
                if (0 != 0) {
                    timeoutTask.cancel();
                }
                throw th;
            }
        }
        return rs;
    }

    private void serverLongData(int parameterIndex, BindValue longData) throws SQLException {
        synchronized (this.connection.getMutex()) {
            MysqlIO mysql = this.connection.getIO();
            Buffer packet = mysql.getSharedSendPacket();
            Object value = longData.value;
            if (value instanceof byte[]) {
                packet.clear();
                packet.writeByte((byte) 24);
                packet.writeLong(this.serverStatementId);
                packet.writeInt(parameterIndex);
                packet.writeBytesNoNull((byte[]) longData.value);
                mysql.sendCommand(24, null, packet, true, null);
            } else if (value instanceof InputStream) {
                storeStream(mysql, parameterIndex, packet, (InputStream) value);
            } else if (value instanceof java.sql.Blob) {
                storeStream(mysql, parameterIndex, packet, ((java.sql.Blob) value).getBinaryStream());
            } else if (value instanceof Reader) {
                storeReader(mysql, parameterIndex, packet, (Reader) value);
            } else {
                throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.18")).append(value.getClass().getName()).append("'").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
            }
        }
    }

    /* JADX WARN: Finally extract failed */
    private void serverPrepare(String sql) throws SQLException {
        synchronized (this.connection.getMutex()) {
            MysqlIO mysql = this.connection.getIO();
            if (this.connection.getAutoGenerateTestcaseScript()) {
                dumpPrepareForTestcase();
            }
            try {
                try {
                    long begin = 0;
                    if (StringUtils.startsWithIgnoreCaseAndWs(sql, "LOAD DATA")) {
                        this.isLoadDataQuery = true;
                    } else {
                        this.isLoadDataQuery = false;
                    }
                    if (this.connection.getProfileSql()) {
                        begin = System.currentTimeMillis();
                    }
                    String characterEncoding = null;
                    String connectionEncoding = this.connection.getEncoding();
                    if (!this.isLoadDataQuery && this.connection.getUseUnicode() && connectionEncoding != null) {
                        characterEncoding = connectionEncoding;
                    }
                    Buffer prepareResultPacket = mysql.sendCommand(22, sql, null, false, characterEncoding);
                    if (this.connection.versionMeetsMinimum(4, 1, 1)) {
                        prepareResultPacket.setPosition(1);
                    } else {
                        prepareResultPacket.setPosition(0);
                    }
                    this.serverStatementId = prepareResultPacket.readLong();
                    this.fieldCount = prepareResultPacket.readInt();
                    this.parameterCount = prepareResultPacket.readInt();
                    this.parameterBindings = new BindValue[this.parameterCount];
                    for (int i = 0; i < this.parameterCount; i++) {
                        this.parameterBindings[i] = new BindValue();
                    }
                    this.connection.incrementNumberOfPrepares();
                    if (this.profileSQL) {
                        this.eventSink.consumeEvent(new ProfilerEvent((byte) 2, "", this.currentCatalog, this.connectionId, this.statementId, -1, System.currentTimeMillis(), mysql.getCurrentTimeNanosOrMillis() - begin, mysql.getQueryTimingUnits(), (String) null, new Throwable(), truncateQueryToLog(sql)));
                    }
                    if (this.parameterCount > 0 && this.connection.versionMeetsMinimum(4, 1, 2) && !mysql.isVersion(MAX_DATE_REP_LENGTH, 0, 0)) {
                        this.parameterFields = new Field[this.parameterCount];
                        int i2 = 0;
                        for (Buffer metaDataPacket = mysql.readPacket(); !metaDataPacket.isLastDataPacket() && i2 < this.parameterCount; metaDataPacket = mysql.readPacket()) {
                            int i3 = i2;
                            i2++;
                            this.parameterFields[i3] = mysql.unpackField(metaDataPacket, false);
                        }
                    }
                    if (this.fieldCount > 0) {
                        this.resultFields = new Field[this.fieldCount];
                        int i4 = 0;
                        for (Buffer fieldPacket = mysql.readPacket(); !fieldPacket.isLastDataPacket() && i4 < this.fieldCount; fieldPacket = mysql.readPacket()) {
                            int i5 = i4;
                            i4++;
                            this.resultFields[i5] = mysql.unpackField(fieldPacket, false);
                        }
                    }
                    this.connection.getIO().clearInputStream();
                } catch (Throwable th) {
                    this.connection.getIO().clearInputStream();
                    throw th;
                }
            } catch (SQLException e) {
                sqlEx = e;
                if (this.connection.getDumpQueriesOnException()) {
                    StringBuffer messageBuf = new StringBuffer(this.originalSql.length() + 32);
                    messageBuf.append("\n\nQuery being prepared when exception was thrown:\n\n");
                    messageBuf.append(this.originalSql);
                    sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf.toString());
                }
                throw sqlEx;
            }
        }
    }

    private String truncateQueryToLog(String sql) {
        String query;
        if (sql.length() > this.connection.getMaxQuerySizeToLog()) {
            StringBuffer queryBuf = new StringBuffer(this.connection.getMaxQuerySizeToLog() + MAX_DATETIME_REP_LENGTH);
            queryBuf.append(sql.substring(0, this.connection.getMaxQuerySizeToLog()));
            queryBuf.append(Messages.getString("MysqlIO.25"));
            query = queryBuf.toString();
        } else {
            query = sql;
        }
        return query;
    }

    private void serverResetStatement() throws SQLException {
        synchronized (this.connection.getMutex()) {
            MysqlIO mysql = this.connection.getIO();
            Buffer packet = mysql.getSharedSendPacket();
            packet.clear();
            packet.writeByte((byte) 26);
            packet.writeLong(this.serverStatementId);
            try {
                try {
                    mysql.sendCommand(26, null, packet, !this.connection.versionMeetsMinimum(4, 1, 2), null);
                    mysql.clearInputStream();
                } catch (Throwable th) {
                    mysql.clearInputStream();
                    throw th;
                }
            } catch (SQLException sqlEx) {
                throw sqlEx;
            } catch (Exception ex) {
                throw SQLError.createSQLException(ex.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
            }
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setArray(int i, Array x) throws SQLException {
        throw new NotImplemented();
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setAsciiStream(int parameterIndex, InputStream x, int length) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, true);
        setType(binding, MysqlDefs.FIELD_TYPE_BLOB);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = true;
        if (this.connection.getUseStreamLengthsInPrepStmts()) {
            binding.bindLength = length;
        } else {
            binding.bindLength = -1L;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, 3);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        if (this.connection.versionMeetsMinimum(MAX_DATE_REP_LENGTH, 0, 3)) {
            setType(binding, 246);
        } else {
            setType(binding, this.stringTypeCode);
        }
        binding.value = StringUtils.fixDecimalExponent(StringUtils.consistentToString(x));
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setBinaryStream(int parameterIndex, InputStream x, int length) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, true);
        setType(binding, MysqlDefs.FIELD_TYPE_BLOB);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = true;
        if (this.connection.getUseStreamLengthsInPrepStmts()) {
            binding.bindLength = length;
        } else {
            binding.bindLength = -1L;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setBlob(int parameterIndex, java.sql.Blob x) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, true);
        setType(binding, MysqlDefs.FIELD_TYPE_BLOB);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = true;
        if (this.connection.getUseStreamLengthsInPrepStmts()) {
            binding.bindLength = x.length();
        } else {
            binding.bindLength = -1L;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setBoolean(int parameterIndex, boolean x) throws SQLException {
        setByte(parameterIndex, x ? (byte) 1 : (byte) 0);
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setByte(int parameterIndex, byte x) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 1);
        binding.value = null;
        binding.byteBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setBytes(int parameterIndex, byte[] x) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 253);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
        checkClosed();
        if (reader == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, true);
        setType(binding, MysqlDefs.FIELD_TYPE_BLOB);
        binding.value = reader;
        binding.isNull = false;
        binding.isLongData = true;
        if (this.connection.getUseStreamLengthsInPrepStmts()) {
            binding.bindLength = length;
        } else {
            binding.bindLength = -1L;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setClob(int parameterIndex, java.sql.Clob x) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, -2);
            return;
        }
        BindValue binding = getBinding(parameterIndex, true);
        setType(binding, MysqlDefs.FIELD_TYPE_BLOB);
        binding.value = x.getCharacterStream();
        binding.isNull = false;
        binding.isLongData = true;
        if (this.connection.getUseStreamLengthsInPrepStmts()) {
            binding.bindLength = x.length();
        } else {
            binding.bindLength = -1L;
        }
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setDate(int parameterIndex, java.sql.Date x) throws SQLException {
        setDate(parameterIndex, x, null);
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setDate(int parameterIndex, java.sql.Date x, Calendar cal) throws SQLException {
        if (x == null) {
            setNull(parameterIndex, 91);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 10);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setDouble(int parameterIndex, double x) throws SQLException {
        checkClosed();
        if (!this.connection.getAllowNanAndInf() && (x == Double.POSITIVE_INFINITY || x == Double.NEGATIVE_INFINITY || Double.isNaN(x))) {
            throw SQLError.createSQLException(new StringBuffer().append("'").append(x).append("' is not a valid numeric or approximate numeric value").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, MAX_DATE_REP_LENGTH);
        binding.value = null;
        binding.doubleBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setFloat(int parameterIndex, float x) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 4);
        binding.value = null;
        binding.floatBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setInt(int parameterIndex, int x) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 3);
        binding.value = null;
        binding.intBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setLong(int parameterIndex, long x) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 8);
        binding.value = null;
        binding.longBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setNull(int parameterIndex, int sqlType) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        if (binding.bufferType == 0) {
            setType(binding, 6);
        }
        binding.value = null;
        binding.isNull = true;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setNull(int parameterIndex, int sqlType, String typeName) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        if (binding.bufferType == 0) {
            setType(binding, 6);
        }
        binding.value = null;
        binding.isNull = true;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setRef(int i, Ref x) throws SQLException {
        throw new NotImplemented();
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setShort(int parameterIndex, short x) throws SQLException {
        checkClosed();
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 2);
        binding.value = null;
        binding.shortBinding = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setString(int parameterIndex, String x) throws SQLException {
        checkClosed();
        if (x == null) {
            setNull(parameterIndex, 1);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, this.stringTypeCode);
        binding.value = x;
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setTime(int parameterIndex, Time x) throws SQLException {
        setTimeInternal(parameterIndex, x, null, this.connection.getDefaultTimeZone(), false);
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
        setTimeInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
    }

    public void setTimeInternal(int parameterIndex, Time x, Calendar targetCalendar, TimeZone tz, boolean rollForward) throws SQLException {
        if (x == null) {
            setNull(parameterIndex, 92);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, 11);
        Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
        synchronized (sessionCalendar) {
            binding.value = TimeUtil.changeTimezone(this.connection, sessionCalendar, targetCalendar, x, tz, this.connection.getServerTimezoneTZ(), rollForward);
        }
        binding.isNull = false;
        binding.isLongData = false;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
        setTimestampInternal(parameterIndex, x, null, this.connection.getDefaultTimeZone(), false);
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
        setTimestampInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
    }

    protected void setTimestampInternal(int parameterIndex, Timestamp x, Calendar targetCalendar, TimeZone tz, boolean rollForward) throws SQLException {
        if (x == null) {
            setNull(parameterIndex, 93);
            return;
        }
        BindValue binding = getBinding(parameterIndex, false);
        setType(binding, MAX_DATETIME_REP_LENGTH);
        Calendar sessionCalendar = this.connection.getUseJDBCCompliantTimezoneShift() ? this.connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
        synchronized (sessionCalendar) {
            binding.value = TimeUtil.changeTimezone(this.connection, sessionCalendar, targetCalendar, x, tz, this.connection.getServerTimezoneTZ(), rollForward);
        }
        binding.isNull = false;
        binding.isLongData = false;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void setType(BindValue oldValue, int bufferType) {
        if (oldValue.bufferType != bufferType) {
            this.sendTypesToServer = true;
        }
        oldValue.bufferType = bufferType;
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException {
        checkClosed();
        throw new NotImplemented();
    }

    @Override // com.mysql.jdbc.PreparedStatement, java.sql.PreparedStatement
    public void setURL(int parameterIndex, URL x) throws SQLException {
        checkClosed();
        setString(parameterIndex, x.toString());
    }

    private void storeBinding(Buffer packet, BindValue bindValue, MysqlIO mysql) throws SQLException {
        try {
            Object value = bindValue.value;
            switch (bindValue.bufferType) {
                case 0:
                case 15:
                case 246:
                case 253:
                case 254:
                    if (value instanceof byte[]) {
                        packet.writeLenBytes((byte[]) value);
                        return;
                    } else if (!this.isLoadDataQuery) {
                        packet.writeLenString((String) value, this.charEncoding, this.connection.getServerCharacterEncoding(), this.charConverter, this.connection.parserKnowsUnicode(), this.connection);
                        return;
                    } else {
                        packet.writeLenBytes(((String) value).getBytes());
                        return;
                    }
                case 1:
                    packet.writeByte(bindValue.byteBinding);
                    return;
                case 2:
                    packet.ensureCapacity(2);
                    packet.writeInt(bindValue.shortBinding);
                    return;
                case 3:
                    packet.ensureCapacity(4);
                    packet.writeLong(bindValue.intBinding);
                    return;
                case 4:
                    packet.ensureCapacity(4);
                    packet.writeFloat(bindValue.floatBinding);
                    return;
                case MAX_DATE_REP_LENGTH /* 5 */:
                    packet.ensureCapacity(8);
                    packet.writeDouble(bindValue.doubleBinding);
                    return;
                case 7:
                case 10:
                case MAX_DATETIME_REP_LENGTH /* 12 */:
                    storeDateTime(packet, (Date) value, mysql);
                    return;
                case 8:
                    packet.ensureCapacity(8);
                    packet.writeLongLong(bindValue.longBinding);
                    return;
                case 11:
                    storeTime(packet, (Time) value);
                    return;
                default:
                    return;
            }
        } catch (UnsupportedEncodingException e) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("ServerPreparedStatement.22")).append(this.connection.getEncoding()).append("'").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    /* JADX WARN: Finally extract failed */
    private void storeDataTime412AndOlder(Buffer intoBuf, Date dt) throws SQLException {
        Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
        synchronized (sessionCalendar) {
            Date oldTime = sessionCalendar.getTime();
            try {
                intoBuf.ensureCapacity(8);
                intoBuf.writeByte((byte) 7);
                sessionCalendar.setTime(dt);
                int year = sessionCalendar.get(1);
                int month = sessionCalendar.get(2) + 1;
                int date = sessionCalendar.get(MAX_DATE_REP_LENGTH);
                intoBuf.writeInt(year);
                intoBuf.writeByte((byte) month);
                intoBuf.writeByte((byte) date);
                if (dt instanceof java.sql.Date) {
                    intoBuf.writeByte((byte) 0);
                    intoBuf.writeByte((byte) 0);
                    intoBuf.writeByte((byte) 0);
                } else {
                    intoBuf.writeByte((byte) sessionCalendar.get(11));
                    intoBuf.writeByte((byte) sessionCalendar.get(MAX_DATETIME_REP_LENGTH));
                    intoBuf.writeByte((byte) sessionCalendar.get(MAX_TIME_REP_LENGTH));
                }
                sessionCalendar.setTime(oldTime);
            } catch (Throwable th) {
                sessionCalendar.setTime(oldTime);
                throw th;
            }
        }
    }

    private void storeDateTime(Buffer intoBuf, Date dt, MysqlIO mysql) throws SQLException {
        if (this.connection.versionMeetsMinimum(4, 1, 3)) {
            storeDateTime413AndNewer(intoBuf, dt);
        } else {
            storeDataTime412AndOlder(intoBuf, dt);
        }
    }

    /* JADX WARN: Finally extract failed */
    private void storeDateTime413AndNewer(Buffer intoBuf, Date dt) throws SQLException {
        Calendar sessionCalendar = ((dt instanceof Timestamp) && this.connection.getUseJDBCCompliantTimezoneShift()) ? this.connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
        synchronized (sessionCalendar) {
            Date oldTime = sessionCalendar.getTime();
            try {
                sessionCalendar.setTime(dt);
                if (dt instanceof java.sql.Date) {
                    sessionCalendar.set(11, 0);
                    sessionCalendar.set(MAX_DATETIME_REP_LENGTH, 0);
                    sessionCalendar.set(MAX_TIME_REP_LENGTH, 0);
                }
                byte length = 7;
                if (dt instanceof Timestamp) {
                    length = 11;
                }
                intoBuf.ensureCapacity(length);
                intoBuf.writeByte(length);
                int year = sessionCalendar.get(1);
                int month = sessionCalendar.get(2) + 1;
                int date = sessionCalendar.get(MAX_DATE_REP_LENGTH);
                intoBuf.writeInt(year);
                intoBuf.writeByte((byte) month);
                intoBuf.writeByte((byte) date);
                if (dt instanceof java.sql.Date) {
                    intoBuf.writeByte((byte) 0);
                    intoBuf.writeByte((byte) 0);
                    intoBuf.writeByte((byte) 0);
                } else {
                    intoBuf.writeByte((byte) sessionCalendar.get(11));
                    intoBuf.writeByte((byte) sessionCalendar.get(MAX_DATETIME_REP_LENGTH));
                    intoBuf.writeByte((byte) sessionCalendar.get(MAX_TIME_REP_LENGTH));
                }
                if (length == 11) {
                    intoBuf.writeLong(((Timestamp) dt).getNanos() / MysqlErrorNumbers.ER_HASHCHK);
                }
                sessionCalendar.setTime(oldTime);
            } catch (Throwable th) {
                sessionCalendar.setTime(oldTime);
                throw th;
            }
        }
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:31:0x0167
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private void storeReader(com.mysql.jdbc.MysqlIO r9, int r10, com.mysql.jdbc.Buffer r11, java.io.Reader r12) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 364
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ServerPreparedStatement.storeReader(com.mysql.jdbc.MysqlIO, int, com.mysql.jdbc.Buffer, java.io.Reader):void");
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:19:0x00fd
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private void storeStream(com.mysql.jdbc.MysqlIO r8, int r9, com.mysql.jdbc.Buffer r10, java.io.InputStream r11) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 258
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ServerPreparedStatement.storeStream(com.mysql.jdbc.MysqlIO, int, com.mysql.jdbc.Buffer, java.io.InputStream):void");
    }

    @Override // com.mysql.jdbc.PreparedStatement
    public String toString() {
        StringBuffer toStringBuf = new StringBuffer();
        toStringBuf.append("com.mysql.jdbc.ServerPreparedStatement[");
        toStringBuf.append(this.serverStatementId);
        toStringBuf.append("] - ");
        try {
            toStringBuf.append(asSql());
        } catch (SQLException sqlEx) {
            toStringBuf.append(Messages.getString("ServerPreparedStatement.6"));
            toStringBuf.append(sqlEx);
        }
        return toStringBuf.toString();
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getServerStatementId() {
        return this.serverStatementId;
    }
}
